﻿using System;
using System.Collections;
using System.Text;

using Cosmos.UserFrame;
using Cosmos.ServiceProvider;
using Cosmos.Common;
using Cosmos.CommonManager;
using System.Data;

/*-----------------------------------------------------------------------------------------------*/
// 설      명 : 해피 쿠폰
// 작  성  자 : 
// 변경  이력 : 
/*-----------------------------------------------------------------------------------------------*/
namespace Cosmos.Service
{
    class PointHappyCoupon : PaymentBase, IPaymentUs
    {

        public PointHappyCoupon()
        {
        }

        #region SearchPayment 결제 조회
        /// <summary>
        /// 결제 조회(인카드 쿠폰 조회)
        /// </summary>
        /// <param name="aParam"></param>
        /// <param name="aRet"></param>
        /// <returns></returns>
        public string SearchPayment(string[] aParam, ref string[] aRet)
        {
            string sRet = UserCom.RST_ERR;

            string sInPutType = "", sInPutData = "", sInEncData = "", sSearchDiv = "";

            try
            {
                sInPutType = aParam[0];                      // 입력구분
                sInPutData = aParam[1];                      // 입력데이터
                sInEncData = aParam[2];                      // 카드데이터
                sSearchDiv = aParam[3];                      // 조회구분(0:해피쿠폰조회, 1:인카드쿠폰조회)

                m_cPayItem = new Column.TR_PAYMENT.DATA();                  // 결과 저장 변수 생성
                m_cPayItem.PAY_WAY_CD = ItemConst.TR_ITEM_ID.COUPON_ITEM;
                m_cPayItem.PAY_DTL_CD_01 = ItemConst.TR_ITEM_ID.COUPON.INCARD;

                if (sSearchDiv == "1")
                {
                    // 인카드쿠폰 조회
                    sRet = SearchInCardCouponIrt(PosConst.CANCEL_DIV.NORMAL, sInPutType, sInPutData, sInEncData);
                }
                else
                {
                    // 해피쿠폰 조회
                    sRet = ExecutePointHappyCouponIrt(PosConst.CANCEL_DIV.NORMAL, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A, "HPY", sInPutType, sInPutData, sInEncData, 0, "", "");
                }
                if (sRet != UserCom.RST_OK) return sRet;

                
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                   System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                CmUtil.ZeroFillClear(ref aParam);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }
        #endregion

        #region GetPayment 결제 정보 획득
        /// <summary>
        /// 결제 정보 획득
        /// </summary>
        /// <param name="aParam"></param>
        /// <returns></returns>
        public object GetPayment(string[] aParam)
        {
            return m_cPayItem;
        }
        #endregion

        #region SetPayment 결제 등록
        /// <summary>
        /// 결제 등록(해피쿠폰 할인)
        /// </summary>
        /// <param name="aParam"></param>
        /// <returns></returns>
        public string SetPayment(string[] aParam)
        {
            string sRet = UserCom.RST_ERR;

            string sInPutType = "", sInPutData = "", sInEncData = "";

            try
            {
                if (m_cTrnStatus.Head.TradeDiv != ItemConst.TRAN_DIV.NORMAL) return sRet;

                sInPutType = aParam[0];                      // 입력구분
                sInPutData = aParam[1];                      // 입력데이터(쿠폰번호)
                sInEncData = aParam[2];                      // 입력데이터(암호화)
                long nDestAmt = CmUtil.LongParse(aParam[3]);        // 할인대상금액
                int nSelRow = CmUtil.IntParse(aParam[4]);           // 상품행번호

                m_cPayItem = new Column.TR_PAYMENT.DATA();      // 결과 저장 변수 생성
                m_cPayItem.PAY_WAY_CD = ItemConst.TR_ITEM_ID.COUPON_ITEM;
                m_cPayItem.PAY_DTL_CD_01 = ItemConst.TR_ITEM_ID.COUPON.INCARD;

                sRet = ExecutePointHappyCouponIrt(PosConst.CANCEL_DIV.NORMAL, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_A, "HPY", sInPutType, sInPutData, sInEncData, nDestAmt, "", "");
                if (sRet != UserCom.RST_OK) return sRet;

                // 쿠폰금액이 받을금액보다 크면 등록 할수 없도록 처리(2017.06.02)
                if (m_cTrnStatus.Sale.RemainPayAmt < m_cPayItem.PAY_AMT)
                {
                    string sApprDate = m_cPayItem.OCCUR_ENTRY_03;           // 승인일자
                    string sApprNo = m_cPayItem.OCCUR_ENTRY_02;             // 승인번호

                    //WinManager.ErrorMessage(MessageManager.GetErrorMessage(POS_MESSAGE.ERROR.MSG_0138) + PosConst.CRLF
                    //    + MessageManager.GetLabelMessage(POS_MESSAGE.LABEL.MSG_0200) + " : " + m_cPosStatus.Global.NumericTOCurrency(m_cPayItem.PAY_AMT));

                    while (true)
                    {
                        sRet = ExecutePointHappyCouponIrt(PosConst.CANCEL_DIV.CANCEL, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_C, "HPY", sInPutType, sInPutData, sInEncData, nDestAmt, sApprDate, sApprNo);
                        if (sRet != UserCom.RST_OK)
                        {
                            //if (WinManager.QuestionMessage(MessageManager.GetErrorMessage(POS_MESSAGE.ERROR.MSG_0221)) == true) continue;

                            return sRet;
                        }
                        break;
                    }
                    return UserCom.RST_ERR;
                }

                if (nSelRow >= 0)
                {
                    // 상품코드 설정
                    ArrayList alSaleItem = (ArrayList)StateObject.GetItemObject(Column.TR_PLU.ITEM);
                    Column.TR_PLU.DATA cPluItem = (Column.TR_PLU.DATA)alSaleItem[nSelRow];
                    m_cPayItem.PAY_DTL_CD_04 = cPluItem.ITEM_PLU_CD;      // 상품코드
                }

                // 결제 아이템 추가
                ArrayList alPayItem = (ArrayList)StateObject.GetItemObject(Column.TR_PAYMENT.ITEM);     // 결제 내역 받아 오기
                m_cPayItem.SEQ = alPayItem.Count + 1;
                alPayItem.Add(m_cPayItem);

                if (m_cPayItem.PAY_DTL_CD_05 == PosConst.PAY_DC_TYPE.DC)
                {
                    SetPluDiscountAmount(nSelRow, m_cPayItem.PAY_WAY_CD + m_cPayItem.PAY_DTL_CD_01, sInPutData, m_cPayItem.PAY_AMT);
                }
                m_cDataService.UpdatePluAmount();       // 상품 합계금액 계산(거래해더)
                m_cDataService.UpdatePayAmount();       // 결제 금액 재 계산 처리
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                   System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                CmUtil.ZeroFillClear(ref aParam);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }
        #endregion

        #region CancelPayment 결제 취소
        /// <summary>
        /// 결제 취소
        /// </summary>
        /// <param name="aParam"></param>
        /// <returns></returns>
        public string CancelPayment(string[] aParam)
        {
            string sRet = UserCom.RST_ERR;

            string sInPutType = "", sInPutData = "", sInEncData = "";

            try
            {
                if (m_cTrnStatus.Head.TradeDiv != ItemConst.TRAN_DIV.NORMAL) return sRet;

                int nPayRow = CmUtil.IntParse(aParam[0]); // 취소할 행번호

                // 결제 내역 받아 오기
                ArrayList alPayItem = (ArrayList)StateObject.GetItemObject(Column.TR_PAYMENT.ITEM);
                m_cPayItem = (Column.TR_PAYMENT.DATA)alPayItem[nPayRow];

                sInPutType = m_cPayItem.OCCUR_ENTRY_05;          // 입력구분
                sInPutData = m_cPayItem.OCCUR_ENTRY_01;          // 입력데이터
                sInEncData = m_cPayItem.CanFiller1;              // 카드데이터(암호화)

                double nDestAmt = m_cPayItem.AMT_ENTRY_10;              // 대상금액
                string sApprDate = m_cPayItem.OCCUR_ENTRY_03;           // 승인일자
                string sApprNo = m_cPayItem.OCCUR_ENTRY_02;             // 승인번호

                sRet = ExecutePointHappyCouponIrt(PosConst.CANCEL_DIV.CANCEL, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_C, "HPY", sInPutType, sInPutData, sInEncData, nDestAmt, sApprDate, sApprNo);
                if (sRet != UserCom.RST_OK) return sRet;

                if (m_cPayItem.PAY_DTL_CD_05 == PosConst.PAY_DC_TYPE.DC)
                {
                    ClearPluDiscountAmount(m_cPayItem.PAY_WAY_CD + m_cPayItem.PAY_DTL_CD_01, sInPutData);
                }
                m_cPayItem.CANCEL_DIV = PosConst.CANCEL_DIV.CANCEL;

                m_cDataService.UpdatePluAmount();       // 상품 합계금액 계산(거래해더)
                m_cDataService.UpdatePayAmount();       // 결제 금액 재 계산 처리

                //WinManager.ConfirmMessage(MessageManager.GetErrorMessage(POS_MESSAGE.ERROR.MSG_0068));

                sRet = UserCom.RST_OK;
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                   System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                CmUtil.ZeroFillClear(ref aParam);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }
        #endregion

        #region RefundPayment 결제 반품
        /// <summary>
        /// 결제 반품
        /// </summary>
        /// <param name="aParam"></param>
        /// <returns></returns>
        public string RefundPayment(string[] aParam)
        {
            string sRet = UserCom.RST_ERR;

            string sInPutType = "", sInPutData = "", sInEncData = "";

            try
            {
                int nPayRow = CmUtil.IntParse(aParam[0]); // 취소할 행번호
                
                // 결제 내역 받아 오기
                ArrayList alPayItem = (ArrayList)StateObject.GetItemObject(Column.TR_PAYMENT.ITEM);
                m_cPayItem = (Column.TR_PAYMENT.DATA)alPayItem[nPayRow];

                sInPutType = m_cPayItem.OCCUR_ENTRY_05;          // 입력구분
                sInPutData = m_cPayItem.OCCUR_ENTRY_01;          // 입력데이터(마스킹)
                sInEncData = m_cPayItem.CanFiller1;              // 카드데이터(암호화)

                if (m_cPayItem.OCCUR_ENTRY_08.ToString() != ItemConst.PAY_APP_DIV.NORMAL)
                {
                    //WinManager.ConfirmMessage(MessageManager.GetErrorMessage(POS_MESSAGE.ERROR.MSG_0240));
                    m_cPayItem.OCCUR_ENTRY_06 = m_cPayItem.OCCUR_ENTRY_02;              // 원승인번호
                    m_cPayItem.OCCUR_ENTRY_07 = m_cPayItem.OCCUR_ENTRY_03;              // 원승인일자
                }
                else
                { 
                    double nDestAmt = m_cPayItem.AMT_ENTRY_10;              // 대상금액
                    string sApprDate = m_cPayItem.OCCUR_ENTRY_03;           // 승인일자
                    string sApprNo = m_cPayItem.OCCUR_ENTRY_02;             // 승인번호

                    sRet = ExecutePointHappyCouponIrt(PosConst.CANCEL_DIV.CANCEL, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_C, "HPY", sInPutType, sInPutData, sInEncData, nDestAmt, sApprDate, sApprNo);
                    if (sRet != UserCom.RST_OK) return sRet;
                }
                sRet = UserCom.RST_OK;
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                   System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                CmUtil.ZeroFillClear(ref aParam);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }
        #endregion

        #region SetMenualPayment
        public string SetMenualPayment(string[] aParam)
        {
            throw new NotImplementedException();
        }
        #endregion

        #region 해피쿠폰 할인 조회 실행
        /// <summary>
        /// 해피쿠폰 인카드 쿠폰 조회 실행
        /// </summary>
        public string SearchInCardCouponIrt(string sTradeDiv, string sInPutType, string sInPutData, string sInEncData)
        {
            string sSendData = "", sRecvData = "";         // 송수신 전문
            int nNowPos = 0;
            string sRet = UserCom.RST_ERR;
            try
            {
                m_cPayItem.HP_CPN_INFO = new ArrayList();
                // 연습모드이면 연습 데이터 설정
                if (m_cPosStatus.Base.TrainingFlag == "1")
                {
                    m_cPayItem.HP_CPN_INFO.Add(new string[] { "10000000001", "쿠폰 테스트1", "20100101", "20201231" });
                    m_cPayItem.HP_CPN_INFO.Add(new string[] { "10000000002", "쿠폰 테스트2", "20100101", "20201231" });
                    m_cPayItem.HP_CPN_INFO.Add(new string[] { "10000000003", "쿠폰 테스트3", "20100101", "20201231" });
                    m_cPayItem.HP_CPN_INFO.Add(new string[] { "10000000004", "쿠폰 테스트4", "20100101", "20201231" });
                    m_cPayItem.HP_CPN_INFO.Add(new string[] { "10000000005", "쿠폰 테스트5", "20100101", "20201231" });
                    return UserCom.RST_OK;
                }

                string sCmpApprID = PosMstManager.GetMstVan(ItemConst.TR_ITEM_ID.COUPON_ITEM, ItemConst.TR_ITEM_ID.COUPON.INCARD, PosMst.MST_VAN.DATA.CMP_APPR_ID);
                string sApprID = PosMstManager.GetMstVan(ItemConst.TR_ITEM_ID.COUPON_ITEM, ItemConst.TR_ITEM_ID.COUPON.INCARD,  PosMst.MST_VAN.DATA.APPR_ID);

                for (int nPageNo = 1; nPageNo < 4; nPageNo++)
                {
                    string sAddInfo = "";
                    // 인카드 쿠폰 조회
                    sAddInfo += CmUtil.RPadH("", 10);                           // 요청 사용자 ID                               
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.StoreNo, 10);    // 점포코드
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.PosNo, 2);       // 포스번호
                    sAddInfo += "10";                                           // 10:인카드쿠폰조회
                    sAddInfo += string.Format("{0:D2}", nPageNo);
                    sAddInfo += CmUtil.RPadH("", 174);                          // VReserved
                    
                    // 포인트 전문 바디 생성                
                    sSendData = MakePointBody("HPY", sInPutType, sInEncData, 0, "", "", "", "", "", "", sAddInfo);

                    // 포인트 전문 헤더 생성
                    sSendData = m_cDataCommon.MakeCreditCardHeader(PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A, sSendData.Length, sCmpApprID, sApprID) + sSendData;

                    // 요청 - 승인로그 저장   (판매구분, [0]결제수단, [1]결제상세코드, [2]전문구분, [3]요청구분, [4]카드번호, [5]결제금액, [6]승인번호, [7]승인일자, [8]승인시간, [9]응답상태값, [10]응답메시지, [11]전문)
                    m_cDataCommon.SetSaleApprLog(sTradeDiv, new string[] { m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A, "S", sInPutData, "0", "", "", "", "", "", sSendData });

                    // 포인트 전문 송수신
                    sRet = m_cDataCommon.ExecuteCardIrt(PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A, sSendData, ref sRecvData, true, sInPutType);
                    if (sRet != UserCom.RST_OK)
                    {
                        //WinManager.ErrorMessage(sRet);
                        return sRet;
                    }

                    byte[] bytes = Encoding.Default.GetBytes(sRecvData);
                    string[] aIrtRsp = new string[Column.POS_RES_HPCOUPON.LEN.Length];

                    ItemColumn.ParseMessage(Column.POS_RES_HPCOUPON.LEN, Column.POS_RES_HPCOUPON.TYPE, bytes, ref nNowPos, ref aIrtRsp);

                    // 응답 - 승인로그 저장   (판매구분, [0]결제수단, [1]결제상세코드, [2]전문구분, [3]요청구분, [4]카드번호, [5]결제금액, [6]승인번호, [7]승인일자, [8]승인시간, [9]응답상태값, [10]응답메시지, [11]전문)
                    m_cDataCommon.SetSaleApprLog(sTradeDiv, new string[] { m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A, "R"
                                                            , sInPutData, ""
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResApprNo], CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResTrDate], 0, 6), CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResTrDate], 6, 6)                                                            
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStatus].Trim()
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResMessage1].Trim() + aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResMessage2].Trim()
                                                            , sRecvData });

                    // - 'O' : 승인 'X' : 거절
                    if (aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStatus] != "O")
                    {
                        sRet = aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResMessage1].Trim() + "\n" + aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResMessage2].Trim();
                        //WinManager.ErrorMessage(sRet);
                        return sRet;
                    }

                    if( CmUtil.LongParse(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponCnt]) >= 1)
                    {
                        m_cPayItem.HP_CPN_INFO.Add(new string[] { aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponNo1], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResOper1] 
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStart1], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStop1]});
                    }
                    if (CmUtil.LongParse(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponCnt]) >= 2)
                    {
                        m_cPayItem.HP_CPN_INFO.Add(new string[] { aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponNo2], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResOper2] 
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStart2], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStop2]});
                    }
                    if (CmUtil.LongParse(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponCnt]) >= 3)
                    {
                        m_cPayItem.HP_CPN_INFO.Add(new string[] { aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponNo3], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResOper3] 
                                                            , aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStart3], aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStop3]});
                    }

                    if (CmUtil.LongParse(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResTotalCnt]) >= m_cPayItem.HP_CPN_INFO.Count || CmUtil.LongParse(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResCouponCnt]) < 3)
                        break;
                }

                sRet = UserCom.RST_OK;
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                    System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }

        /// <summary>
        /// 해피쿠폰 할인 조회 실행
        /// </summary>
        public string ExecutePointHappyCouponIrt(string sTradeDiv, string sTrType, string sPointType, string sInPutType, string sInPutData, string sInEncData, double nDestAmt, string sApprDate, string sApprNo)
        {
            string sRet = UserCom.RST_ERR;
            try
            {
                // 연습모드이면 연습 데이터 설정
                if (m_cPosStatus.Base.TrainingFlag == "1" || m_cPosStatus.Mst.TestStorYn == "1" ) return SetTrainingData(sTrType, sPointType, sInPutType, sInPutData, sInEncData, nDestAmt, sApprDate, sApprNo);

                string sAddInfo = "";
                if (sTrType == PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_A || sTrType == PosConst.POS_VAN_MASTER.TRADE_ID.VAN_POT_USE_C)
                {
                    // 사용,사용취소
                    sAddInfo += "Y";                                            // 쿠폰거래유무
                    sAddInfo += CmUtil.RPadH("", 10);                           // 요청 사용자 ID
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.SaleDate, 8);    // POS 영업일자 - POS 매출일자 8자리
                    sAddInfo += "2001";                                         // 거래사유코드 - 2001:대금결제
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.SaleDate, 10);   // SAP 코드 - POS 매출일자 10자리
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.PosNo, 2);       // POS 번호
                    sAddInfo += string.Format("{0:D10}", (long)m_cTrnStatus.Sale.NetSaleAmt);        // 사용대상거래금액
                    sAddInfo += CmUtil.RPadH("", 20);                           // 결제카드번호 - 신용 결제한 카드번호
                    sAddInfo += CmUtil.RPadH("", 135); ;                        // VReserved
                }
                else
                {
                    // 쿠폰 조회
                    sAddInfo += CmUtil.RPadH("", 10);                           // 요청 사용자 ID                               
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.StoreNo, 10);    // 점포코드
                    sAddInfo += CmUtil.RPadH(m_cPosStatus.Base.PosNo, 2);       // 포스번호
                    sAddInfo += "20";                                           // 10:인카드쿠폰조회, 20:쿠폰정보조회
                    sAddInfo += string.Format("{0:D2}", 0);
                    sAddInfo += CmUtil.RPadH("", 174);                          // VReserved
                }

                string sVanCD = GetPayDtlCdToVanCode(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);    // 승인업체 코드 조회
                if (sVanCD == "")
                {
                    //WinManager.ErrorMessage(POS_MESSAGE.ERROR.MSG_0563);
                    return sRet;
                }

                // 포인트 조회
                string[] aIrtRsp = null;
                sRet = ExecutePointIrt(sTradeDiv, m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, sVanCD, sTrType, sPointType, sInPutType, sInPutData, sInEncData, nDestAmt, sApprDate, sApprNo, "", "", "", "", sAddInfo, ref aIrtRsp, "", "");
                if (sRet != UserCom.RST_OK) return sRet;

                if (sTradeDiv == PosConst.CANCEL_DIV.NORMAL)
                {
                    m_cPayItem.PAY_AMT = CmUtil.DoubleParse(aIrtRsp[Column.POS_RES_POINT.SEQ.ResTrPoint]);          // 결제금액

                    m_cPayItem.PAY_DTL_CD_03 = "0";                                 // 할인금액아이템적용여부 (0:미적용, 1:적용)
                    m_cPayItem.PAY_DTL_CD_05 = PosConst.PAY_DC_TYPE.PAY;            // 할인결제구분 (1:결제, 2:할인)
                    if (PosMstManager.GetMstPayDc(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, PosMst.MST_PAY_DC.DATA.PAY_DC_TYPE) == PosConst.PAY_DC_TYPE.DC)
                    {
                        m_cPayItem.PAY_DTL_CD_03 = "1";                             // 할인금액아이템적용여부 (0:미적용, 1:적용)
                        m_cPayItem.PAY_DTL_CD_05 = PosConst.PAY_DC_TYPE.DC;         // 할인결제구분 (1:결제, 2:할인)
                    }

                    m_cPayItem.QTY_ENTRY_01 = 1;                    // 수량

                    m_cPayItem.AMT_ENTRY_01 = m_cPayItem.PAY_AMT;   // 받은금액
                    m_cPayItem.AMT_ENTRY_02 = m_cPayItem.PAY_AMT;   // 할인금액(쿠폰금액)
                    m_cPayItem.AMT_ENTRY_10 = nDestAmt;             // 대상상품금액

                    m_cPayItem.OCCUR_ENTRY_01 = sInPutData;         // 카드번호
                    m_cPayItem.OCCUR_ENTRY_02 = aIrtRsp[Column.POS_RES_POINT.SEQ.ResApprNo].Trim();                 // 승인번호
                    m_cPayItem.OCCUR_ENTRY_03 = CmUtil.MidH(aIrtRsp[Column.POS_RES_POINT.SEQ.ResTrDate], 0, 6);     // 승인일자
                    m_cPayItem.OCCUR_ENTRY_04 = CmUtil.MidH(aIrtRsp[Column.POS_RES_POINT.SEQ.ResTrDate], 6, 4);     // 승인시간
                    m_cPayItem.OCCUR_ENTRY_05 = sInPutType;         // 입력 구분
                    m_cPayItem.OCCUR_ENTRY_06 = sApprNo;            // 원승인번호
                    m_cPayItem.OCCUR_ENTRY_07 = sApprDate;          // 원승인일자
                    m_cPayItem.OCCUR_ENTRY_08 = ItemConst.PAY_APP_DIV.NORMAL;       //승인구분
                    m_cPayItem.OCCUR_ENTRY_16 = sVanCD;             // VAN 구분
                    m_cPayItem.OCCUR_ENTRY_18 = aIrtRsp[Column.POS_RES_POINT.SEQ.ResStoreNo];                       // 가맹점번호
                    m_cPayItem.OCCUR_ENTRY_20 = GetPayDtlCdToPayDtlName(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);   // 결제수단명
                    m_cPayItem.OCCUR_ENTRY_21 = GetPayDtlCdToCashBillYn(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);   // 현금영수증 발행여부

                    m_cPayItem.CANCEL_DIV = ItemConst.PAY_CANCEL_DIV.NORMAL;
                    m_cPayItem.BILLSPR_NO = m_cTrnStatus.Sale.BillSplitNo;  // 빌분리 번호

                    m_cPayItem.CanFiller1 = sInEncData;                     // 암호화된 카드 데이터
                    m_cPayItem.CanFiller2 = aIrtRsp[Column.POS_RES_POINT.SEQ.ResNotice1];   // 쿠폰명


                    //#20171116 온라인 쿠폰 제품군코드,상품코드 존재 시 할인 대상이 변경되도록 추가 start, phj
                    if (sTrType == PosConst.POS_VAN_MASTER.TRADE_ID.VAN_HPY_ELC_A)
                    {
                        m_cPayItem.OCCUR_ENTRY_02 = aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResApprNo].Trim();                 // 승인번호
                        m_cPayItem.OCCUR_ENTRY_03 = CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResTrDate], 0, 6);     // 승인일자
                        m_cPayItem.OCCUR_ENTRY_04 = CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResTrDate], 6, 4);     // 승인시간
                        m_cPayItem.OCCUR_ENTRY_18 = aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResStoreNo].Trim();                       // 가맹점번호
                        m_cPayItem.CanFiller2 = CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResOper1], 0, 40).Trim();         // 쿠폰명


                        // 20171207 start,phj
                        //기존 
                        //#20180502 온라인쿠폰 제품군, 상품코드 할인 적용 기능 적용 start
                        //적용시 주석 해제
                        m_cPayItem.CpnItemDcCode = CmUtil.MidH(aIrtRsp[Column.POS_RES_HPCOUPON.SEQ.ResOper1], 40, 10).Trim();     // 쿠폰 조회 응답의 제품군 코드 또는 상품코드
                        //#20180502 온라인쿠폰 제품군, 상품코드 할인 적용 기능 적용 end

                        //변경 
                        //12월 중에 40,10 바이트 분리해서 내려주는 부분이 해피에서 작업이 안되어있기 때문에 임시로 변경
                        //#20180502 온라인쿠폰 제품군, 상품코드 할인 적용 기능 적용 end

                        //적용시 주석
                        //m_cPayItem.CpnItemDcCode = "";     // 쿠폰 조회 응답의 제품군 코드 또는 상품코드
                        //#20180502 온라인쿠폰 제품군, 상품코드 할인 적용 기능 적용 end
                        // 20171207 start,phj

                    }
                    //#20171116 온라인 쿠폰 제품군코드,상품코드 존재 시 할인 대상이 변경되도록 추가 end, phj
                }
                else
                {
                    m_cPayItem.OCCUR_ENTRY_01 = sInPutData;         // 카드번호
                    m_cPayItem.OCCUR_ENTRY_02 = aIrtRsp[Column.POS_RES_POINT.SEQ.ResApprNo].Trim();                 // 승인번호
                    m_cPayItem.OCCUR_ENTRY_03 = CmUtil.MidH(aIrtRsp[Column.POS_RES_POINT.SEQ.ResTrDate], 0, 6);     // 승인일자
                    m_cPayItem.OCCUR_ENTRY_04 = CmUtil.MidH(aIrtRsp[Column.POS_RES_POINT.SEQ.ResTrDate], 6, 4);     // 승인시간
                    m_cPayItem.OCCUR_ENTRY_05 = sInPutType;         // 입력 구분
                    m_cPayItem.OCCUR_ENTRY_06 = sApprNo;            // 원승인번호
                    m_cPayItem.OCCUR_ENTRY_07 = sApprDate;          // 원승인일자
                }
                sRet = UserCom.RST_OK;
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                    System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }

        /// <summary>
        /// 연습 데이터 설정
        /// </summary>
        public string SetTrainingData(string sTrType, string sPointType, string sInPutType, string sInPutData, string sInEncData, double nDestAmt, string sApprDate, string sApprNo)
        {
            string sRet = UserCom.RST_ERR;
            try
            {
                // 연습모드이면 연습 데이터 설정
                string sVanCD = GetPayDtlCdToVanCode(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);    // 승인업체 코드 조회
                if (sVanCD == "")
                {
                    WinManager.ErrorMessage(POS_MESSAGE.ERROR.MSG_0563);
                    return sRet;
                }

                double nDisAmt = m_cDataCommon.ExecTermCoopCmpPayDc(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, "", nDestAmt);
                if(nDisAmt == 0) nDisAmt = CmUtil.MathRounds(CmUtil.MathAmtPercent(nDestAmt, 50), m_cPosStatus.Mst.DcRudDwLocMethd, CmUtil.IntParse(m_cPosStatus.Mst.DcRudDwLoc)); // 50% 할인

                m_cPayItem.PAY_AMT = nDisAmt;               // 결제금액

                m_cPayItem.PAY_DTL_CD_03 = "0";                                 // 할인금액아이템적용여부 (0:미적용, 1:적용)
                m_cPayItem.PAY_DTL_CD_05 = PosConst.PAY_DC_TYPE.PAY;            // 할인결제구분 (1:결제, 2:할인)
                if (PosMstManager.GetMstPayDc(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01, PosMst.MST_PAY_DC.DATA.PAY_DC_TYPE) == PosConst.PAY_DC_TYPE.DC)
                {
                    m_cPayItem.PAY_DTL_CD_03 = "1";                             // 할인금액아이템적용여부 (0:미적용, 1:적용)
                    m_cPayItem.PAY_DTL_CD_05 = PosConst.PAY_DC_TYPE.DC;         // 할인결제구분 (1:결제, 2:할인)
                }

                m_cPayItem.QTY_ENTRY_01 = 1;                    // 수량

                m_cPayItem.AMT_ENTRY_01 = m_cPayItem.PAY_AMT;   // 받은금액
                m_cPayItem.AMT_ENTRY_02 = m_cPayItem.PAY_AMT;   // 할인금액(쿠폰금액)
                m_cPayItem.AMT_ENTRY_10 = nDestAmt;             // 대상상품금액

                m_cPayItem.OCCUR_ENTRY_01 = sInPutData;         // 카드번호
                m_cPayItem.OCCUR_ENTRY_02 = "99999999";                         // 승인번호
                m_cPayItem.OCCUR_ENTRY_03 = DateTime.Now.ToString("yyMMdd");    // 승인일자
                m_cPayItem.OCCUR_ENTRY_04 = DateTime.Now.ToString("HHmm");      // 승인시간
                m_cPayItem.OCCUR_ENTRY_05 = sInPutType;         // 입력 구분
                m_cPayItem.OCCUR_ENTRY_06 = sApprNo;            // 원승인번호
                m_cPayItem.OCCUR_ENTRY_07 = sApprDate;          // 원승인일자
                m_cPayItem.OCCUR_ENTRY_08 = ItemConst.PAY_APP_DIV.COMPULSION;       //승인구분
                m_cPayItem.OCCUR_ENTRY_16 = sVanCD;             // VAN 구분
                m_cPayItem.OCCUR_ENTRY_20 = GetPayDtlCdToPayDtlName(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);   // 결제수단명
                m_cPayItem.OCCUR_ENTRY_21 = GetPayDtlCdToCashBillYn(m_cPayItem.PAY_WAY_CD, m_cPayItem.PAY_DTL_CD_01);   // 현금영수증 발행여부

                m_cPayItem.CANCEL_DIV = ItemConst.PAY_CANCEL_DIV.NORMAL;
                m_cPayItem.BILLSPR_NO = m_cTrnStatus.Sale.BillSplitNo;  // 빌분리 번호

                m_cPayItem.CanFiller1 = sInEncData;                     // 암호화된 카드 데이터

                sRet = UserCom.RST_OK;
            }
            catch (Exception ex)
            {
                WinManager.ExceptionMessage(System.Reflection.Assembly.GetExecutingAssembly().ManifestModule.Name,
                    System.Reflection.MethodBase.GetCurrentMethod().DeclaringType.Name + "." + System.Reflection.MethodBase.GetCurrentMethod().Name + "()", ex.Message);
            }
            finally
            {
                //여전법 대응!
                //////////////////////////////////////////////////////////////////////////////////
                CmUtil.ZeroFillClear(ref sInPutData);
                CmUtil.ZeroFillClear(ref sInEncData);
                //////////////////////////////////////////////////////////////////////////////////
            }
            return sRet;
        }
        #endregion

    }
}
